@@ -1,12 +1,12 @@
 #============================================================================
 #  Function: target_explore_cat
 #============================================================================
-#' Explore categorial variable + target
+#' Explore categorical variable + target
 #'
-#' Create a plot to explore relation between categorial variable and a binary target
+#' Create a plot to explore relation between categorical variable and a binary target
 #'
 #' @param data A dataset
-#' @param var Categorial variable
+#' @param var Categorical variable
 #' @param target Target variable (0/1 or FALSE/TRUE)
 #' @param min_val All values < min_val are converted to min_val
 #' @param max_val All values > max_val are converted to max_val
@@ -151,7 +151,7 @@ target_explore_cat <- function(data, var, target = "target_ind", min_val = NA, m
 #============================================================================
 #  Function: target_explore_num
 #============================================================================
-#' Explore categorial variable + target
+#' Explore categorical variable + target
 #'
 #' Create a plot to explore relation between numerical variable and a binary target
 #'
@@ -260,9 +260,9 @@ target_explore_num <- function(data, var, target = "target_ind", min_val = NA, m
 #============================================================================
 #  explore_bar
 #============================================================================
-#' Explore categorial variable using bar charts
+#' Explore categorical variable using bar charts
 #'
-#' Create a barplot to explore a categorial variable.
+#' Create a barplot to explore a categorical variable.
 #' If a target is selected, the barplot is created for all levels of the target.
 #'
 #' @param data A dataset
@@ -661,6 +661,7 @@ explore_density <- function(data, var, target, title = "", min_val = NA, max_val
 #' Explore all variables of a dataset (create plots)
 #'
 #' @param data A dataset
+#' @param n Weights variable (only for count data)
 #' @param target Target variable (0/1 or FALSE/TRUE)
 #' @param ncol Layout of plots (number of columns)
 #' @param split Split by target (TRUE|FALSE)
@@ -674,7 +675,7 @@ explore_density <- function(data, var, target, title = "", min_val = NA, max_val
 #' explore_all(iris, target = is_virginica)
 #' @export
 
-explore_all <- function(data, target, ncol = 2, split = TRUE)  {
+explore_all <- function(data, n, target, ncol = 2, split = TRUE)  {
 
   # check parameter data
   assertthat::assert_that(!missing(data), msg = "expect a data table to explore")
@@ -691,7 +692,15 @@ explore_all <- function(data, target, ncol = 2, split = TRUE)  {
     guess_target = "oth"
   }
 
-  # varable name of target
+  # parameter n
+  if(!missing(n))  {
+    n_quo <- enquo(n)
+    n_txt <- quo_name(n_quo)[[1]]
+  } else {
+    n_txt = NA
+  }
+
+  # variable name of target
   var_name_target = target_txt
 
   # names of variables in data
@@ -702,6 +711,11 @@ explore_all <- function(data, target, ncol = 2, split = TRUE)  {
     var_names <- var_names[var_names != var_name_target]
   }
 
+  # if n is used, ignore n variable
+  if (!is.na(n_txt)) {
+    var_names <- var_names[var_names != n_txt]
+  }
+
   #pre define list of plots
   plots <- list(mode = "list", length = length(var_names))
 
@@ -714,19 +728,33 @@ explore_all <- function(data, target, ncol = 2, split = TRUE)  {
     var_name <- var_names[i]
 
     # reduce variables of data (to improve speed and memory)
-    if (is.na(var_name_target)) {
+    if (is.na(var_name_target) & is.na(n_txt)) {
       data_tmp <- data[var_name]
     }
-    else {
+    else if (!is.na(var_name_target) & is.na(n_txt)) {
       data_tmp <- data[c(var_name, var_name_target)]
     }
+    else if (is.na(var_name_target) & !is.na(n_txt)) {
+      data_tmp <- data[c(var_name, n_txt)]
+    }
+    else if (!is.na(var_name_target) & !is.na(n_txt)) {
+      data_tmp <- data[c(var_name, n_txt, var_name_target)]
+    }
 
     # intelligent guessing if num or cat
     # based on postfix and type of variable names
     var_type <- guess_cat_num(data_tmp[[var_name]])
 
+    # count data, no target
+    if (!is.na(n_txt) & (is.na(var_name_target)))  {
+      plots[[i]] <- explore_count(data_tmp, n = !!n_quo)
+
+    # count data, target
+    } else if (!is.na(n_txt) & (!is.na(var_name_target)))  {
+        plots[[i]] <- explore_count(data_tmp, n = !!n_quo, target = !!target_quo, split = split)
+
     # no target, num
-    if ( (var_type == "num") & (is.na(var_name_target))) {
+    } else if ( (var_type == "num") & (is.na(var_name_target))) {
       plots[[i]] <- explore_density(data_tmp, !!sym(var_name))
 
       # no target, cat
@@ -826,10 +854,15 @@ explore_cor <- function(data, x, y, target, bins = 8, min_val = NA, max_val = NA
     target_txt = NA
   }
 
-  # guess cat/num
-  x_type = guess_cat_num(data[[x_txt]])
-  y_type = guess_cat_num(data[[y_txt]])
+  # describe x, y
+  x_descr <- data %>% describe(!!x_quo, out = "list")
+  y_descr <- data %>% describe(!!y_quo, out = "list")
 
+  # guess cat/num x,y
+  x_type = guess_cat_num(data[[x_txt]], descr = x_descr)
+  y_type = guess_cat_num(data[[y_txt]], descr = y_descr)
+
+  # guess cat/num target
   if (!is.na(target_txt)) {
     target_type = guess_cat_num(data[[target_txt]])
   }
@@ -837,6 +870,7 @@ explore_cor <- function(data, x, y, target, bins = 8, min_val = NA, max_val = NA
     target_type = "oth"
   }
 
+  # auto_scale?
   if(x_type == "num")  {
 
     # autoscale (if mni_val and max_val not used)
@@ -852,7 +886,34 @@ explore_cor <- function(data, x, y, target, bins = 8, min_val = NA, max_val = NA
 
   } # if num
 
-  if(x_type == "num" & y_type == "num")  {
+  # use geom_point?
+  use_points <- ifelse(
+      x_type == "num" & y_type == "num" &
+      nrow(data) <= 500 &
+      x_descr$unique / nrow(data) >= 0.1 &
+      y_descr$unique / nrow(data) >= 0.1,
+      TRUE,
+      FALSE
+  )
+
+  if(x_type == "num" & y_type == "num" & use_points)  {
+
+    if (!is.na(target_txt)) {
+
+    p <- data %>%
+      ggplot(aes(x = !!x_quo, y = !!y_quo, color = !!target_quo)) +
+      geom_point(alpha = 0.45, size = 2.5) +
+      theme_light()
+    } else {
+
+      p <- data %>%
+        ggplot(aes(x = !!x_quo, y = !!y_quo)) +
+        geom_point(alpha = 0.45, size = 2.5) +
+        theme_light()
+    }
+  }
+
+  else if(x_type == "num" & y_type == "num" & !use_points)  {
 
     # boxplot (x = num, y = num)
     p <- data %>%
@@ -860,9 +921,8 @@ explore_cor <- function(data, x, y, target, bins = 8, min_val = NA, max_val = NA
       ggplot(aes(x = !!x_quo, y = !!y_quo)) +
       geom_boxplot(aes(group = cut(!!x_quo, bins)), fill = color) +
       theme_light()
-
-
   }
+
   else if(x_type == "cat" & y_type == "num") {
 
     data[[x_txt]] <- as.factor(data[[x_txt]])
@@ -897,11 +957,11 @@ explore_cor <- function(data, x, y, target, bins = 8, min_val = NA, max_val = NA
       theme_light()
   }
 
-  if(!is.na(target_txt) & (target_type == "cat")) {
+  # facet
+  if(!is.na(target_txt) & (target_type == "cat") & !use_points) {
     p <- p + facet_grid(vars(!!target_quo))
   }
 
-
   # title
   if (!is.na(title) & nchar(title) > 0)  {
     p <- p + ggtitle(title)
@@ -920,13 +980,14 @@ explore_cor <- function(data, x, y, target, bins = 8, min_val = NA, max_val = NA
 #' Explore a table. Plots variable types, variables with no variance and variables with NA
 #'
 #' @param data A dataset
+#' @param n Weight variable for count data
 #' @importFrom magrittr "%>%"
 #' @import dplyr
 #' @examples
 #' explore_tbl(iris)
 #' @export
 
-explore_tbl <- function(data)  {
+explore_tbl <- function(data, n)  {
 
   # define variables to pass CRAN-checks
   type <- NULL
@@ -938,6 +999,28 @@ explore_tbl <- function(data)  {
   assertthat::assert_that(is.data.frame(data), msg = "expect a table of type data.frame")
   assertthat::assert_that(nrow(data) > 0, msg = "data has 0 observations")
 
+  # parameter n
+  if (!missing(n)) {
+    n_quo <- enquo(n)
+    n_txt <- quo_name(n_quo)[[1]]
+    if (!n_txt %in% names(data))  {
+      stop(paste0("variable '", n_txt, "' not found"))
+    }
+    info_obs <- paste("with", format_num_kMB(nrow(data)), "observations")
+  } else {
+    n_quo <- NA
+    n_txt <- NA
+  }
+
+  # info text for observations
+  if (is.na(n_txt)) {
+    info_obs <- paste0("with ", format_num_kMB(nrow(data)), " observations")
+  } else {
+    total_nrow <- sum(data[[n_txt]])
+    info_obs <- paste0("with ", format_num_kMB(total_nrow), " observations (",
+                      nrow(data), " in raw data)")
+  }
+
   # describe data
   d <- describe_all(data)
 
@@ -999,7 +1082,7 @@ explore_tbl <- function(data)  {
               position = "stack"
               ) +
     labs(title = paste(ncol(data), "variables"),
-         subtitle = paste("with", format_num_kMB(nrow(data)), "observations"),
+         subtitle = info_obs,
          y = "variables",
          x = "") +
     coord_flip() +
@@ -1154,7 +1237,7 @@ explore_shiny <- function(data, target)  {
       shiny::removeModal()
 
       # show Report
-      browseURL(paste0("file://", file.path(output_dir, output_file)))
+      browseURL(paste0("file://", file.path(output_dir, output_file)), browser = NULL)
     })
 
     output$graph_target <- shiny::renderPlot({
@@ -1221,6 +1304,7 @@ explore_shiny <- function(data, target)  {
 #' @param data A dataset
 #' @param var A variable
 #' @param var2 A variable for checking correlation
+#' @param n A Variable for number of observations (count data)
 #' @param target Target variable (0/1 or FALSE/TRUE)
 #' @param split Split by target variable (FALSE/TRUE)
 #' @param min_val All values < min_val are converted to min_val
@@ -1260,7 +1344,7 @@ explore_shiny <- function(data, target)  {
 #'
 #' @export
 
-explore <- function(data, var, var2, target, split, min_val = NA, max_val = NA, auto_scale = TRUE, na = NA, ...)  {
+explore <- function(data, var, var2, n, target, split, min_val = NA, max_val = NA, auto_scale = TRUE, na = NA, ...)  {
 
   # check parameter data
   assertthat::assert_that(!missing(data), msg = "expect a data table to explore")
@@ -1286,12 +1370,23 @@ explore <- function(data, var, var2, target, split, min_val = NA, max_val = NA,
     if (!var2_text %in% names(data))  {
       stop(paste0("variable '", var2_text, "' not found"))
     }
-
   } else {
     var2_quo <- NA
     var2_text <- NA
   }
 
+  # parameter n
+  if (!missing(n)) {
+    n_quo <- enquo(n)
+    n_text <- quo_name(n_quo)[[1]]
+    if (!n_text %in% names(data))  {
+      stop(paste0("variable '", n_text, "' not found"))
+    }
+  } else {
+    n_quo <- NA
+    n_text <- NA
+  }
+
   # parameter target
   if(!missing(target))  {
     target_quo <- enquo(target)
@@ -1324,6 +1419,14 @@ explore <- function(data, var, var2, target, split, min_val = NA, max_val = NA,
   if (is.na(var_text))  {
     explore_shiny(data)
 
+    # count data
+  } else if (!is.na(n_text) & is.na(target_text))  {
+    explore_count(data, cat = !!var_quo, n = !!n_quo, split = split, ...)
+
+    # count data + target
+  } else if (!is.na(n_text)  & !is.na(target_text))  {
+    explore_count(data, cat = !!var_quo, n = !!n_quo, target = !!target_quo, split = split, ...)
+
     # var + var2 -> correlation
   } else if (!is.na(var_text) & !is.na(var2_text) & is.na(target_text))  {
     explore_cor(data[unique(c(var_text, var2_text))],
@@ -1512,3 +1615,168 @@ explore_targetpct <- function(data, var, target = NULL, title = NULL, min_val =
 
 } # explore_targetpct
 
+#============================================================================
+#  Function: explore_count
+#============================================================================
+#' Explore count data (categories + frequency)
+#'
+#' Create a plot to explore count data (categories + freuency)
+#' Variable named 'n' is auto detected as Frequency
+#'
+#' @param data A dataset (categories + frequency)
+#' @param cat Numerical variable
+#' @param n Number of observations (frequency)
+#' @param target Target variable
+#' @param pct Show as percent?
+#' @param split Split by target?
+#' @param title Title of the plot
+#' @param flip Flip plot? (for categorical variables)
+#' @return Plot object
+#' @examples
+#' library(dplyr)
+#' iris %>%
+#'   count(Species) %>%
+#'   explore_count(Species)
+#' @importFrom magrittr "%>%"
+#' @import rlang
+#' @export
+
+explore_count <- function(data, cat, n, target, pct = FALSE, split = TRUE, title = NA, flip = NA)  {
+
+  # define variables for CRAN-package check
+  plot_cat <- NULL
+  plot_n <- NULL
+  plot_target <- NULL
+  plot_n_sum <- NULL
+  plot_n_pct <- NULL
+  plot_n_tot <- NULL
+
+  # check parameters
+  assertthat::assert_that(!missing(data), msg = "expect a data table to explore")
+  assertthat::assert_that(is.data.frame(data), msg = "expect a table of type data.frame")
+  assertthat::assert_that(nrow(data) > 0, msg = "data has 0 observations")
+  assertthat::assert_that(ncol(data) >= 2, msg = "explect at least 2 variables in data")
+
+  # parameter var
+  if(!missing(cat))  {
+    cat_quo <- enquo(cat)
+    cat_txt <- quo_name(cat_quo)[[1]]
+    if (!cat_txt %in% names(data)) {
+      stop(paste0("variable '", cat_txt, "' not found"))
+    }
+  } else {
+    cat_txt <- names(data)[1]
+  }
+
+  # parameter var
+  if(!missing(n))  {
+    n_quo <- enquo(n)
+    n_txt <- quo_name(n_quo)[[1]]
+    if (!n_txt %in% names(data)) {
+      stop(paste0("variable '", n_txt, "' not found"))
+    }
+  } else {
+    if("n" %in% names(data)) {
+      n_txt <- "n"
+    } else {
+      stop("variable n not defined")
+    }
+  }
+
+  # parameter var
+  if(!missing(target))  {
+    target_quo <- enquo(target)
+    target_txt <- quo_name(target_quo)[[1]]
+    if (!target_txt %in% names(data)) {
+      stop(paste0("variable '", target_txt, "' not found"))
+    }
+  } else {
+    target_txt <- ""
+  }
+
+  ## no target defined
+  if(target_txt == "")  {
+
+    # prepare data (no target)
+    data_plot <- data[, c(cat_txt, n_txt)]
+    names(data_plot) <- c("plot_cat","plot_n")
+    data_plot <- suppressMessages(
+      data_plot %>%
+        group_by(plot_cat) %>%
+        summarise(plot_n_sum = sum(plot_n)) %>%
+        mutate(plot_n_tot = sum(plot_n_sum)) %>%
+        mutate(plot_n_pct = plot_n_sum / plot_n_tot * 100)
+    )
+
+    # geom_col (absolute | percent) no target
+    if(pct == FALSE)  {
+      p <- data_plot %>%
+        ggplot(aes(x = plot_cat, y = plot_n_sum)) +
+        geom_col(position = "dodge", color = "lightgrey", fill = "lightgrey") +
+        theme_light() +
+        labs(y = "count", x = cat_txt)
+    } else {
+      p <- data_plot %>%
+        ggplot(aes(x = plot_cat, y = plot_n_pct)) +
+        geom_col(position = "dodge", color = "lightgrey", fill = "lightgrey") +
+        theme_light() +
+        labs(y = "%", x = cat_txt)
+    }
+
+    ## target defined
+  } else {
+
+    # prepare data (target)
+    data_plot <- data[, c(target_txt, cat_txt, n_txt)]
+    names(data_plot) <- c("plot_target", "plot_cat","plot_n")
+    data_plot <- suppressMessages(
+      data_plot %>%
+        group_by(plot_target, plot_cat) %>%
+        summarise(plot_n_sum = sum(plot_n)) %>%
+        mutate(plot_n_tot = sum(plot_n_sum)) %>%
+        mutate(plot_n_pct = plot_n_sum / plot_n_tot * 100)
+    )
+
+    # geom_col (absolute | percent) with target
+    if(pct == TRUE | split == TRUE)  {
+      # geom_col dodge
+      p <- data_plot %>%
+        ggplot(aes(x = plot_cat, y = plot_n_pct, fill = plot_target)) +
+        geom_col(position = "dodge") +
+        theme_light() +
+        labs(y = "%", x = cat_txt, fill = target_txt)
+    } else {
+      # geom_col stack
+      p <- data_plot %>%
+        ggplot(aes(x = plot_cat, y = plot_n_sum, fill = plot_target)) +
+        geom_col(position = "stack") +
+        theme_light() +
+        labs(y = "count", x = cat_txt, fill = target_txt)
+    }
+  } # if target
+
+
+  # guess flip
+  if (missing(flip)) {
+    if(is.numeric(data[[cat_txt]])) {
+      flip <- FALSE
+    } else {
+      flip <- TRUE
+    }
+  }
+
+  # flip plot
+  if (is.na(flip) | flip) {
+    p <- p + coord_flip()
+  }
+
+  # title
+  if (!is.na(title) & nchar(title) > 0)  {
+    p <- p + ggtitle(title)
+  }
+
+  # return plot
+  p
+  #data_plot
+
+} # explore_count
